import sys
sys.path.append(".")         # can find version in . by pytest
sys.path.append("./cronvice")  # can find version from Jupyter
#from version import __version__
# try:
#     from cronvice.version import __version__
# except:
#    print("")
